/*
*     Projekt Diplom-Pruefungsordnung
*     Relationale Datenbanken
*     Sommersemester 2000
*/

/*
*     Tabellen-Schema
*     12. Juni 2000
*     HMS, Harald M. Schwarz
*/


/*
*   Adress-Typen
*   
*   Z.B.:  Ladungs-Adresse, Privat-Adresse, Dienst-Adresse
*   
*   !!!  Achtung hier aendert sich evtl. bald was  !!!
*   Ich denke nochmal darueber nach. HMS
*/


DROP   TABLE   typ_adressen;
CREATE TABLE   typ_adressen
(
     adr_typ   CHAR(8)  NOT NULL,
     adr_txt   CHAR(25) NOT NULL,

     PRIMARY KEY  (adr_typ)
);


/*
*    Hier wird der Status einer Pruefung bestimmt:
*   
*    Werte koennen z.B. sein:
*    angemeldet; bestanden; nicht bestanden; ausgefallen, entschuldigt;
*    etc.
*/

DROP   TABLE typ_prfstatus;
CREATE TABLE typ_prfstatus
(
    status_typ CHAR(8)             NOT NULL,
    status_txt CHAR(30)            NOT NULL,
    status_gewertet_bestanden      BOOLEAN NOT NULL,
    status_gewertet_nichtbestanden BOOLEAN NOT NULL,

    PRIMARY KEY     (status_typ)
);

/*
*  Hiermit werden die unterschiedlichen Qualifikationen festgelegt
*
*  Abitur, dazu Vergleichbares, etc...
*  Scheine, Seminare, Praktika
*
*/

DROP   TABLE  typ_qualifikate;
CREATE TABLE  typ_qualifikate
(
   qal_typ CHAR(8)  NOT NULL,
   qal_grp CHAR(20),
   qal_txt CHAR(80) NOT NULL,

   PRIMARY KEY  (qal_typ, qal_grp, qal_txt)
);



/*
*  Hiermit werden die unterschiedlichen Pruefungstypen festgelegt
*
*  Vordiplom Info I und II, Mathe, Nebenfaecher, Hauptdiplom A, ...
*
*/

DROP   TABLE  typ_pruefung;
CREATE TABLE  typ_pruefung
(
    prf_typ    CHAR(12)    NOT NULL,
--    prf_num    INTEGER    NOT NULL,
    prf_txt    CHAR(100)  NOT NULL,

    PRIMARY KEY  (prf_typ)
);


/*
*  Hiermit werden die unterschiedlichen Noten-Bewertungen festgelegt
*
* 1.0, 1.1, 1.3  sehr gut
* 1.8, 1.9, 2.0  gut
* etc
*
*/

DROP   TABLE  typ_notentxt;
CREATE TABLE  typ_notentxt
(
    note_typ    FLOAT     NOT NULL,
    note_txt    CHAR(20)  NOT NULL,

    PRIMARY KEY  (note_typ)
);


/*
*    Hiermit wird fuer jedes gueltige Semester 
*    ein Text als Bezeichner angegeben
*    z.B. WS1999/2000
*   
*    Dadurch ist einheitliche Darstellung gesichert.
*/
DROP   TABLE  typ_semestertxt;
CREATE TABLE  typ_semestertxt
(
    semester_txt  CHAR(12)  NOT NULL,

    PRIMARY KEY   (semester_txt) 
);

/*
*   Hiermit wird die Art des Studiums bestimmt:
*
*   Vollzeit, Gasthoerer, Urlaubssemester, ...
*/
DROP   TABLE  typ_studium;
CREATE TABLE  typ_studium
(
    studium_typ  CHAR(8)  NOT NULL,
    studium_txt  CHAR(20) NOT NULL,

    PRIMARY KEY  (studium_typ)
);

/*
*    Die Personalien der Studenten:
*
*    stud_id       Eindeutige Zuordnung, Matrikel-Nummer
*    name          Familienname 
*    vorname       Vorname   ( !! weitere Namen nicht beruecksichtigt !! )   
*    m_w           Unterscheidung maennlich / weiblich
*    geb_datum     Geburtsdatum  
*    geb_ort       Geburtsort
*
*
*    immat_datum   Datum der Einschreibung
*    exmat_datum   Datum der Exmatrikulation
*
*    semester      !! ?? aktuelle Semesterzahl ?? !!
*
*/


DROP   TABLE  pers_studenten;
CREATE TABLE  pers_studenten
(
    stud_id     CHAR(8)  NOT NULL,

    name        CHAR(25) NOT NULL,
    vorname     CHAR(25) NOT NULL,
    m_w         CHAR(1)  NOT NULL,
    geb_datum   DATE     NOT NULL,
    geb_ort     CHAR(25) NOT NULL, 
    
    immat_datum DATE     NOT NULL,
    exmat_datum DATE,

--    semester    INTEGER,
    nebenfach   CHAR(50),

    PRIMARY KEY (stud_id)    

);


/*
*  Die Personalien der Pruefer
*
*    pers_id       Eindeutige Zuordnung,  Personalnummer
*
*    name          Familienname 
*    vorname       Vorname   ( !! weitere Namen nicht beruecksichtigt !! )   
*    m_w           Unterscheidung maennlich / weiblich
*    titel         Prof.,  Dr., Dipl.-Informatiker, etc. ...     
*/

DROP   TABLE  pers_pruefer;
CREATE TABLE  pers_pruefer
(
    pers_id     CHAR(8)  NOT NULL,

    name        CHAR(25) NOT NULL,
    vorname     CHAR(25) NOT NULL,
    m_w         CHAR(1)  NOT NULL,
    titel       CHAR(25) NOT NULL,   
    rang        CHAR(2),   

    einstellung_datum   DATE,
    pensionierung_datum DATE,

    -- Einstellungs- und Pensionierungsdatum?
    -- Prfungsgebiete

    PRIMARY KEY (pers_id)
);



/*
*    Adressen von Studenten und Pruefern
*    
*    id          Zuordnung zur Person
*    adr_typ     Ladungs-Adresse, Privat- oder Dienstadresse, etc...
*    
*    sonstiges   Adr.-Zusaetze, z.B.: "bei Mueller", "Abt. III"
*    raum        Buero oder Stud-Zimmer
*     
*    weiteres sollte selbsterklaerend sein
*/

/*
Achtung die IDs von Studenten und Pruefern duerfen sich nicht decken!
*/

DROP   TABLE   pers_adressen;
CREATE TABLE   pers_adressen
(
    id         CHAR(8)   NOT NULL,
    zaehler    INTEGER   NOT NULL,

    adr_typ    CHAR(8),

    sonstiges  CHAR(25),
    raum       CHAR(25),

    strasse    CHAR(25),
    land       CHAR(3),
    plz        CHAR(5),
    ort        CHAR(25),

    telefon1   CHAR(25),
    telefon2   CHAR(25),
    fax        CHAR(25),
    email      CHAR(50),


   CONSTRAINT adr_typ_exists
       FOREIGN KEY (adr_typ) REFERENCES typ_adressen(adr_typ),
    PRIMARY KEY (id, zaehler)
);



/*
TO DO:
Wie verwalten wir den Studienweg des Studenten?
Fuer jedes Semester einen Eintrag mit 
Einschreibungen, Beurlaubungen, Auslandsaufenthalten, etc?

Wie berechnen wir die Semesterzahl?
Problem: Quereinsteiger von anderen Unis, etc...
*/

DROP   TABLE  uni_studium;
CREATE TABLE  uni_studium
(
     stud_id      CHAR(8)   NOT NULL,
     semester     CHAR(12)  NOT NULL,
     studien_typ  CHAR(8)   NOT NULL,

     PRIMARY KEY  (stud_id, semester),

/*
*    Mit den folgenden CONTRAINTS wird sichergestell, dass
*    a) der Student in der PA-DB gefuehrt wird,
*    b) das Semester gueltig ist,
*    c) der Studientyp gueltig ist.
*/
     CONSTRAINT stud_id_exists
       FOREIGN KEY (stud_id) REFERENCES pers_studenten ON DELETE CASCADE,

     CONSTRAINT semester_exists
       FOREIGN KEY (semester) REFERENCES typ_semestertxt(semester_txt),

     CONSTRAINT studien_typ_exists
       FOREIGN KEY (studien_typ) REFERENCES typ_studium(studium_typ)
);


/*
*    Qualifikationen
*/

DROP   TABLE  uni_qualifikate;
CREATE TABLE  uni_qualifikate
(
    stud_id      CHAR(8)   NOT NULL,
    qal_typ      CHAR(8)   NOT NULL,
    pers_id      CHAR(8),

--    zaehler      INTEGER   NOT NULL,

    datum        DATE      NOT NULL,

    titel        CHAR(100),
    bemerkung    CHAR(100),
    
    PRIMARY KEY  (stud_id, qal_typ, datum, titel),

    CONSTRAINT stud_id_exists
      FOREIGN KEY (stud_id) REFERENCES pers_studenten ON DELETE CASCADE,
/*
    CONSTRAINT qual_typ_exists
       FOREIGN KEY (qal_typ) REFERENCES typ_qualifikate,
*/
    CONSTRAINT pers_id_exists
       FOREIGN KEY (pers_id) REFERENCES pers_pruefer

);


/*
*    Hier werden alle angemeldeten Pruefungen erfasst
*
*    stud_id     Kennung des Studierenden der die Pruefung ablegt
*    typ_id      Kennung der Pruefung (z.B. VDP-Info I und II)
*    versuch     Zaehlung der Pruefungsversuche des Studenten fuer typ_id
*    zaehler     Fuer einen Versuch sind mehrere Anlaeufe moeglich
*                (Abmeldung wegen Krankheit etc....)
*    
*    anmelde_datum   Wann hat sich der Studierende angemeldet?   
*    gew_pruefer     Wen wuenscht der Studierende als Pruefer?
*
*    pruefer_id      Kennung des Pruefers
*    beisitz_id      Kennung des Beisitzers
*
*    status_id       Angemeldet, bestanden, nicht bestanden, abgemeldet, ...
*    datum           Tag der Pruefung
*    note            Ergebnis der Pruefung
*/


DROP   TABLE  uni_pruefungen;
CREATE TABLE  uni_pruefungen
(
    stud_id     CHAR(8)    NOT NULL,
    typ_id      CHAR(12)   NOT NULL,
    versuch     INTEGER    NOT NULL,
    zaehler     INTEGER    NOT NULL,

    anmelde_datum DATE     NOT NULL,
    gew_pruefer_id CHAR(8) NOT NULL,

    pruefer_id  CHAR(8),
    beisitz_id  CHAR(8),  

    status_id   CHAR(8)    NOT NULL, 
    datum       DATE,                   /* NULL-WERT MOEGLICH */
    note        FLOAT,
    bemerkung   CHAR(100),

    thema       CHAR(100) DEFAULT 'Kein Thema',

    PRIMARY KEY (stud_id, typ_id, versuch, zaehler),

    CONSTRAINT stud_id_exists
      FOREIGN KEY (stud_id) REFERENCES pers_studenten ON DELETE CASCADE,

    CONSTRAINT gew_pruefer_id_exists
      FOREIGN KEY (gew_pruefer_id) REFERENCES pers_pruefer(pers_id),

    CONSTRAINT pruefer_id_exists
      FOREIGN KEY (pruefer_id) REFERENCES pers_pruefer(pers_id),

    CONSTRAINT beisitz_id_exists
      FOREIGN KEY (beisitz_id) REFERENCES pers_pruefer(pers_id),

    CONSTRAINT erst_anmelden
      CHECK (anmelde_datum < datum), 
  
    CHECK  (pruefer_id <> beisitz_id),

    CONSTRAINT status_id_exists
      FOREIGN KEY (status_id) REFERENCES typ_prfstatus,

    CONSTRAINT note_exists
      FOREIGN KEY (note) REFERENCES typ_notentxt(note_typ)


);


/*
*    Angaben zu Diplom-Arbeiten
*    
*    Aehnlich wie bei obigen Pruefungen
*
*    !! Hier muss ich nochmal die verschieden Daten mit der DPO
*    abstimmen !!
*    
*/


DROP   TABLE  uni_diplarbeit;
CREATE TABLE  uni_diplarbeit
(
    stud_id        CHAR(8) NOT NULL,
    versuch        INTEGER NOT NULL,
    zaehler        INTEGER NOT NULL,

    anmelde_datum  DATE    NOT NULL,
    gew_pruefer_id CHAR(8) NOT NULL,
 
    thema          CHAR(100),
    ausgabe_datum  DATE,    
    abgabe_datum   DATE,
    abgabe_limit   DATE,

    pruefer1_id    CHAR(8),
    pruefer2_id    CHAR(8),
    status_id      CHAR(8),
    note           FLOAT,
    bemerkung      CHAR(100),

    PRIMARY KEY    (stud_id, versuch, zaehler),

    CONSTRAINT stud_id_exists
      FOREIGN KEY (stud_id) REFERENCES pers_studenten ON DELETE CASCADE,

    CONSTRAINT gew_pruefer_id_exists
      FOREIGN KEY (gew_pruefer_id) REFERENCES pers_pruefer(pers_id),

    CONSTRAINT pruefer1_id_exists
      FOREIGN KEY (pruefer1_id) REFERENCES pers_pruefer(pers_id),

    CONSTRAINT pruefer2_id_exists
      FOREIGN KEY (pruefer2_id) REFERENCES pers_pruefer(pers_id),
  
    CHECK  (pruefer1_id <> pruefer2_id),

    CONSTRAINT status_id_exists
      FOREIGN KEY (status_id) REFERENCES typ_prfstatus,

    CONSTRAINT note_exists
      FOREIGN KEY (note) REFERENCES typ_notentxt(note_typ)
  
);


