-- ##########################################################################
-- ####                                                                  ####
-- ####                         Prfungsamt-Projekt                      ####
-- ####                    ============================                  ####
-- ####                                                                  ####
-- #### Sichten                                                          ####
-- ####                                                                  ####
-- #### Version 1.00  --  12. Juli 2000                                  ####
-- ####                                                                  ####
-- #### Copyright (C) 2000  Thomas Dreibholz                             ####
-- ####                     Universitt Bonn                             ####
-- ####                     EMail: Dreibholz@bigfoot.com                 ####
-- ####                     WWW:   http://www.bigfoot.com/~dreibholz     ####
-- ####                                                                  ####
-- ##########################################################################


-- ##########################################################################
-- #### Prfer-Views                                                     ####
-- ##########################################################################


-- ====== Immatrikulierte Studenten =========================================
DROP VIEW pers_pruefer_np;
CREATE VIEW pers_pruefer_np AS
   SELECT *
   FROM pers_pruefer p
   WHERE p.pensionierung_datum IS NULL;


-- ====== Exmatrikulierte Studenten =========================================
DROP VIEW pers_pruefer_pe;
CREATE VIEW pers_pruefer_pe AS
   SELECT *
   FROM pers_pruefer p
   WHERE p.pensionierung_datum IS NOT NULL;



-- ##########################################################################
-- #### Prfungen-Views                                                  ####
-- ##########################################################################


-- ====== Bestandene Prfungen ==============================================
DROP VIEW uni_pruefungen_best;
CREATE VIEW uni_pruefungen_best AS
   SELECT * FROM uni_pruefungen p
   WHERE EXISTS (SELECT * FROM typ_prfstatus t
                 WHERE p.status_id = t.status_typ AND
                       t.status_gewertet_bestanden = TRUE);


-- ====== Nicht bestandene Prfungen ========================================
DROP VIEW uni_pruefungen_nbest;
CREATE VIEW uni_pruefungen_nbest AS
   SELECT * FROM uni_pruefungen p
   WHERE EXISTS (SELECT * FROM typ_prfstatus t
                 WHERE p.status_id = t.status_typ AND
                       t.status_gewertet_nichtbestanden = TRUE);


-- ====== Nicht gewertete Prfungen =========================================
DROP VIEW uni_pruefungen_ngew;
CREATE VIEW uni_pruefungen_ngew AS
   SELECT * FROM uni_pruefungen p
   WHERE EXISTS (SELECT * FROM typ_prfstatus t
                 WHERE p.status_id                      = t.status_typ AND
                       t.status_gewertet_bestanden      = FALSE        AND
                       t.status_gewertet_nichtbestanden = FALSE);


-- ====== Vordiplom-Prfungen ===============================================
DROP VIEW uni_pruefungen_vd;
CREATE VIEW uni_pruefungen_vd AS
   SELECT * FROM uni_pruefungen p
   WHERE p.typ_id IN ('VD-I/II','VD-III/IV','VD-MATH','VD-NF');


-- ====== Hauptdiplom-Prfungen =============================================
DROP VIEW uni_pruefungen_hd;
CREATE VIEW uni_pruefungen_hd AS
   SELECT * FROM uni_pruefungen p
   WHERE p.typ_id IN ('HD-A','HD-B','HD-C','HD-NF');



-- ##########################################################################
-- #### Diplomarbeiten-Views                                             ####
-- ##########################################################################


-- ====== Bestandene Diplomarbeit ===========================================
DROP VIEW uni_diplarbeit_best;
CREATE VIEW uni_diplarbeit_best AS
   SELECT * FROM uni_diplarbeit d
   WHERE EXISTS (SELECT * FROM typ_prfstatus t
                 WHERE d.status_id = t.status_typ AND
                       t.status_gewertet_bestanden = TRUE);


-- ====== Nicht bestandene Diplomarbeiten ===================================
DROP VIEW uni_diplarbeit_nbest;
CREATE VIEW uni_diplarbeit_nbest AS
   SELECT * FROM uni_diplarbeit d
   WHERE EXISTS (SELECT * FROM typ_prfstatus t
                 WHERE d.status_id = t.status_typ AND
                       t.status_gewertet_nichtbestanden = TRUE);


-- ====== Nicht gewertete Diplomarbeiten ====================================
DROP VIEW uni_diplarbeit_ngew;
CREATE VIEW uni_diplarbeit_ngew AS
   SELECT * FROM uni_diplarbeit d
   WHERE EXISTS (SELECT * FROM typ_prfstatus t
                 WHERE d.status_id                      = t.status_typ AND
                       t.status_gewertet_bestanden      = FALSE        AND
                       t.status_gewertet_nichtbestanden = FALSE);


-- ##########################################################################
-- #### Studenten-Views                                                  ####
-- ##########################################################################


-- ====== Studenten im Grundstudium =========================================
DROP VIEW pers_studenten_gs;
CREATE VIEW pers_studenten_gs AS
   SELECT *
   FROM pers_studenten s
   WHERE s.exmat_datum IS NULL AND
         NOT EXISTS (SELECT q.qal_typ
                     FROM uni_qualifikate q
                     WHERE q.stud_id = s.stud_id AND
                           q.qal_typ = 'VDIPL');


-- ====== Studenten im Hauptstudium =========================================
DROP VIEW pers_studenten_hs;
CREATE VIEW pers_studenten_hs AS
   SELECT *
   FROM pers_studenten s
   WHERE s.exmat_datum IS NULL AND
         EXISTS (SELECT q.qal_typ
                 FROM uni_qualifikate q
                 WHERE q.stud_id = s.stud_id AND
                       q.qal_typ = 'VDIPL');


-- ====== Diplomanden =======================================================
DROP VIEW pers_studenten_dipl;
CREATE VIEW pers_studenten_dipl AS
   SELECT *
   FROM pers_studenten s
   WHERE s.exmat_datum IS NULL AND
         EXISTS (SELECT d.stud_id
                 FROM uni_diplarbeit d
                 WHERE d.stud_id = s.stud_id);


-- ====== Geprfte Studenten ================================================
DROP VIEW pers_studenten_gepr;
CREATE VIEW pers_studenten_gepr AS
   SELECT *
   FROM pers_studenten s
   WHERE s.exmat_datum IS NULL AND
         EXISTS (SELECT p.stud_id
                 FROM uni_pruefungen p, typ_prfstatus t
                 WHERE p.stud_id                   = s.stud_id   AND
                       t.status_typ                = p.status_id AND
                       t.status_gewertet_bestanden = TRUE);


-- ====== Ungeprfte Studenten ==============================================
DROP VIEW pers_studenten_ungepr;
CREATE VIEW pers_studenten_ungepr AS
   SELECT *
   FROM pers_studenten s
   WHERE s.exmat_datum IS NULL AND
         NOT EXISTS (SELECT p.stud_id
                     FROM uni_pruefungen p, typ_prfstatus t
                     WHERE p.stud_id                   = s.stud_id   AND
                           t.status_typ                = p.status_id AND
                           t.status_gewertet_bestanden = TRUE);


-- ====== Langzeit-Studenten ================================================
DROP VIEW pers_studenten_lz;
CREATE VIEW pers_studenten_lz AS
   SELECT *
   FROM pers_studenten s
   WHERE s.exmat_datum IS NULL AND
         (SELECT COUNT(x.semester)
          FROM uni_studium x
          WHERE x.stud_id = s.stud_id) > 9;


-- ====== Immatrikulierte Studenten =========================================
DROP VIEW pers_studenten_ex;
CREATE VIEW pers_studenten_ex AS
   SELECT *
   FROM pers_studenten s
   WHERE s.exmat_datum IS NOT NULL;


-- ====== Exmatrikulierte Studenten =========================================
DROP VIEW pers_studenten_im;
CREATE VIEW pers_studenten_im AS
   SELECT *
   FROM pers_studenten s
   WHERE s.exmat_datum IS NULL;



-- ##########################################################################
-- #### Allgemeine Funktionen                                            ####
-- ##########################################################################


DROP FUNCTION quote(INTEGER,INTEGER);
CREATE FUNCTION quote(INTEGER,INTEGER) RETURNS FLOAT AS '
   DECLARE
      quote FLOAT;
   BEGIN
      IF ($1 + $2) <> 0 THEN
         quote = ROUND((100.0 * $1) / ($1 + $2),2);
      ELSE
         quote = 0.0;
      END IF;
      RETURN quote;
   END;
' LANGUAGE 'plpgsql';


DROP FUNCTION prozent(INTEGER,INTEGER);
CREATE FUNCTION prozent(INTEGER,INTEGER) RETURNS FLOAT AS '
   DECLARE
      prozent FLOAT;
   BEGIN
      IF $2 <> 0 THEN
         prozent = ROUND((100.0 * $1) / $2,1);
      ELSE
         prozent = 0.0;
      END IF;
      RETURN prozent;
   END;
' LANGUAGE 'plpgsql';


DROP FUNCTION division(INTEGER,INTEGER);
CREATE FUNCTION division(INTEGER,INTEGER) RETURNS FLOAT AS '
   DECLARE
      ergebnis FLOAT;
   BEGIN
      IF $2 <> 0 THEN
         ergebnis = ROUND(1.0 * $1 / $2,2);
      ELSE
         ergebnis = NULL;
      END IF;
      RETURN ergebnis;
   END;
' LANGUAGE 'plpgsql';



-- ##########################################################################
-- #### Prfer-Statistik                                                 ####
-- ##########################################################################


-- ====== Anzahl der bestandenen Prfungen ==================================
DROP FUNCTION stat_pruefer_prfbest(CHAR(8));
CREATE FUNCTION stat_pruefer_prfbest(CHAR(8)) RETURNS INTEGER AS '
   DECLARE
      pruefungen RECORD;
      anzahl     INTEGER;
   BEGIN
      SELECT INTO anzahl COUNT(p.pruefer_id)
         FROM uni_pruefungen p, typ_prfstatus s
         WHERE p.pruefer_id                = $1           AND
               p.status_id                 = s.status_typ AND
               s.status_gewertet_bestanden = TRUE;
      RETURN(anzahl);
   END;
' LANGUAGE 'plpgsql';


-- ====== Anzahl der nicht bestandenen Prfungen ============================
DROP FUNCTION stat_pruefer_prfnbest(CHAR(8));
CREATE FUNCTION stat_pruefer_prfnbest(CHAR(8)) RETURNS INTEGER AS '
   DECLARE
      pruefungen RECORD;
      anzahl     INTEGER;
   BEGIN
      SELECT INTO anzahl COUNT(p.pruefer_id)
         FROM uni_pruefungen p, typ_prfstatus s
         WHERE p.pruefer_id                     = $1           AND
               p.status_id                      = s.status_typ AND
               s.status_gewertet_nichtbestanden = TRUE;
      RETURN(anzahl);
   END;
' LANGUAGE 'plpgsql';


-- ====== Anzahl der beantragten Prfungen ==================================
DROP FUNCTION stat_pruefer_prfgesamt(CHAR(8));
CREATE FUNCTION stat_pruefer_prfgesamt(CHAR(8)) RETURNS INTEGER AS '
   DECLARE
      pruefungen RECORD;
      anzahl     INTEGER;
   BEGIN
      SELECT INTO anzahl COUNT(p.pruefer_id)
         FROM uni_pruefungen p
         WHERE p.pruefer_id = $1;
      RETURN(anzahl);
   END;
' LANGUAGE 'plpgsql';


-- ====== Anzahl der bestandenen Prfungen ==================================
DROP FUNCTION stat_pruefer_avgnote(CHAR(8));
CREATE FUNCTION stat_pruefer_avgnote(CHAR(8)) RETURNS FLOAT AS '
   DECLARE
      pruefungen RECORD;
      avgnote    FLOAT;
   BEGIN
      SELECT INTO avgnote ROUND(AVG(p.note),2)
         FROM uni_pruefungen p, typ_prfstatus s
         WHERE p.pruefer_id                = $1           AND
               p.status_id                 = s.status_typ AND
               (s.status_gewertet_bestanden      = TRUE OR
                s.status_gewertet_nichtbestanden = TRUE);
      IF avgnote IS NULL THEN
         SELECT INTO avgnote 0;
      END IF;
      RETURN(avgnote);
   END;
' LANGUAGE 'plpgsql';


-- ====== Hilfs-View ========================================================
DROP VIEW stat_pruefer_prf_tmp;
CREATE VIEW stat_pruefer_prf_tmp AS
   SELECT p.name,
          p.vorname,
          p.pers_id,
          stat_pruefer_prfbest(p.pers_id)   AS bestanden,
          stat_pruefer_prfnbest(p.pers_id)  AS durchgefallen,
          stat_pruefer_prfgesamt(p.pers_id) AS gesamt,
          stat_pruefer_avgnote(p.pers_id)   AS avgnote
   FROM pers_pruefer p
   WHERE p.rang IN ('C1','C2','C3','C4');


-- ====== Prfer-Prfungen-Statistik ========================================
DROP VIEW stat_pruefer_prf;
CREATE VIEW stat_pruefer_prf AS
   SELECT name,
          vorname,
          pers_id,
          lpad(gesamt,4,' ')                               AS gesamt,
          lpad(bestanden,4,' ')                            AS B,
          lpad(durchgefallen,4,' ')                        AS NB,
          lpad(gesamt - bestanden - durchgefallen,4,' ')   AS NG,
          to_char(avgnote,'9.99')                          AS avgnote,
          to_char(quote(durchgefallen,bestanden),'999.99') AS dquote
   FROM stat_pruefer_prf_tmp;


-- ====== Hilfs-View ========================================================
DROP VIEW stat_pruefer_qal_tmp;
CREATE VIEW stat_pruefer_qal_tmp AS
   SELECT p.pers_id,
          q.qal_typ,
          COUNT(q.stud_id) AS qanzahl
   FROM pers_pruefer p,
        uni_qualifikate q
   WHERE p.pers_id = q.pers_id
   GROUP BY p.pers_id,
            q.qal_typ;


-- ====== Prfer-Qualifikate-Statistik ======================================
DROP VIEW stat_pruefer_qal;
CREATE VIEW stat_pruefer_qal AS
   SELECT p.name,
          p.vorname,
          p.pers_id,
          pq.qal_typ,
          lpad(pq.qanzahl,5,' ') AS qanzahl
   FROM pers_pruefer p,
        stat_pruefer_qal_tmp pq
   WHERE p.pers_id = pq.pers_id;




-- ##########################################################################
-- #### Studenten-Statistik                                              ####
-- ##########################################################################


-- ====== Anzahl der bestandenen Prfungen ==================================
DROP FUNCTION stat_studenten_prfbest(CHAR(8));
CREATE FUNCTION stat_studenten_prfbest(CHAR(8)) RETURNS INTEGER AS '
   DECLARE
      pruefungen RECORD;
      anzahl     INTEGER;
   BEGIN
      SELECT INTO anzahl COUNT(p.stud_id)
         FROM uni_pruefungen p, typ_prfstatus s
         WHERE p.stud_id                   = $1           AND
               p.status_id                 = s.status_typ AND
               s.status_gewertet_bestanden = TRUE;
      RETURN(anzahl);
   END;
' LANGUAGE 'plpgsql';


-- ====== Anzahl der nicht bestandenen Prfungen ============================
DROP FUNCTION stat_studenten_prfnbest(CHAR(8));
CREATE FUNCTION stat_studenten_prfnbest(CHAR(8)) RETURNS INTEGER AS '
   DECLARE
      pruefungen RECORD;
      anzahl     INTEGER;
   BEGIN
      SELECT INTO anzahl COUNT(p.stud_id)
         FROM uni_pruefungen p, typ_prfstatus s
         WHERE p.stud_id                        = $1           AND
               p.status_id                      = s.status_typ AND
               s.status_gewertet_nichtbestanden = TRUE;
      RETURN(anzahl);
   END;
' LANGUAGE 'plpgsql';


-- ====== Anzahl der beantragten Prfungen ==================================
DROP FUNCTION stat_studenten_prfgesamt(CHAR(8));
CREATE FUNCTION stat_studenten_prfgesamt(CHAR(8)) RETURNS INTEGER AS '
   DECLARE
      pruefungen RECORD;
      anzahl     INTEGER;
   BEGIN
      SELECT INTO anzahl COUNT(p.stud_id)
         FROM uni_pruefungen p
         WHERE p.stud_id = $1;
      RETURN(anzahl);
   END;
' LANGUAGE 'plpgsql';


-- ====== Anzahl der bestandenen Prfungen ==================================
DROP FUNCTION stat_studenten_avgnote(CHAR(8));
CREATE FUNCTION stat_studenten_avgnote(CHAR(8)) RETURNS FLOAT AS '
   DECLARE
      pruefungen RECORD;
      avgnote    FLOAT;
   BEGIN
      SELECT INTO avgnote ROUND(AVG(p.note),2)
         FROM uni_pruefungen p, typ_prfstatus s
         WHERE p.stud_id                   = $1           AND
               p.status_id                 = s.status_typ AND
               s.status_gewertet_bestanden = TRUE;
      IF avgnote IS NULL THEN
         SELECT INTO avgnote 0;
      END IF;
      RETURN(avgnote);
   END;
' LANGUAGE 'plpgsql';


-- ====== Anzahl der Semester ===============================================
DROP FUNCTION stat_studenten_semester(CHAR(8));
CREATE FUNCTION stat_studenten_semester(CHAR(8)) RETURNS FLOAT AS '
   DECLARE
      pruefungen RECORD;
      semester   INTEGER;
   BEGIN
      SELECT INTO semester COUNT(s.stud_id)
         FROM uni_studium s
         WHERE s.stud_id = $1;
      RETURN(semester);
   END;
' LANGUAGE 'plpgsql';


-- ====== Hilfs-View ========================================================
DROP VIEW stat_stud_prf_tmp;
CREATE VIEW stat_stud_prf_tmp AS
   SELECT s.name,
          s.vorname,
          s.stud_id,
          stat_studenten_semester(s.stud_id)  AS semester,
          stat_studenten_prfbest(s.stud_id)   AS bestanden,
          stat_studenten_prfnbest(s.stud_id)  AS durchgefallen,
          stat_studenten_prfgesamt(s.stud_id) AS gesamt,
          stat_studenten_avgnote(s.stud_id)   AS avgnote
   FROM pers_studenten s;


-- ====== Prfer-Prfungen-Statistik ========================================
DROP VIEW stat_stud_prf;
CREATE VIEW stat_stud_prf AS
   SELECT name,
          vorname,
          stud_id,
          lpad(semester,3,' ')                             AS semester,
          lpad(gesamt,4,' ')                               AS gesamt,
          lpad(bestanden,4,' ')                            AS B,
          lpad(durchgefallen,4,' ')                        AS NB,
          lpad(gesamt - bestanden - durchgefallen,4,' ')   AS NG,
          to_char(avgnote,'9.99')                          AS avgnote,
          to_char(quote(durchgefallen,bestanden),'999.99') AS dquote
   FROM stat_stud_prf_tmp;


-- ====== Hilfs-View ========================================================
DROP VIEW stat_stud_qal_tmp;
CREATE VIEW stat_stud_qal_tmp AS
   SELECT s.stud_id,
          lpad(stat_studenten_semester(s.stud_id),3,' ') AS semester,
          COUNT(q.pers_id)                               AS qanzahl
   FROM pers_studenten s,
        uni_qualifikate q
   WHERE s.stud_id = q.stud_id
   GROUP BY s.stud_id;


-- ====== Prfer-Qualifikate-Statistik ======================================
DROP VIEW stat_stud_qal;
CREATE VIEW stat_stud_qal AS
   SELECT s.name,
          s.vorname,
          s.stud_id,
          lpad(pq.semester,3,' ')                          AS semester,
          lpad(pq.qanzahl,3,' ')                           AS qanzahl,
          to_char(division(pq.qanzahl,pq.semester),'9.99') AS qps
   FROM pers_studenten s,
        stat_stud_qal_tmp pq
   WHERE s.stud_id = pq.stud_id;




-- ##########################################################################
-- #### Anzahl Prfer/Studenten-Statistik                                ####
-- ##########################################################################


-- ====== Anzahl immatrikulierter Studenten =================================
DROP FUNCTION stat_studenten_anzahl(TIMESTAMP);
CREATE FUNCTION stat_studenten_anzahl(TIMESTAMP) RETURNS INTEGER AS '
   DECLARE
      anzahl INTEGER;
   BEGIN
      IF $1 IS NULL THEN
         SELECT INTO anzahl COUNT(s.stud_id)
            FROM pers_studenten s
            WHERE s.exmat_datum IS NULL;
      ELSE
         SELECT INTO anzahl COUNT(s.stud_id)
            FROM pers_studenten s
            WHERE to_char(s.immat_datum,''YYYY'') <= to_char($1,''YYYY'') AND
                  (s.exmat_datum IS NULL OR
                   to_char(s.exmat_datum,''YYYY'') >= to_char($1,''YYYY''));
      END IF;
      RETURN anzahl;
   END;
' LANGUAGE 'plpgsql';


-- ====== Anzahl der nicht-pensionierten Prfer =============================
DROP FUNCTION stat_pruefer_anzahl(TIMESTAMP);
CREATE FUNCTION stat_pruefer_anzahl(TIMESTAMP) RETURNS INTEGER AS '
   DECLARE
      anzahl INTEGER;
   BEGIN
      IF $1 IS NULL THEN
         SELECT INTO anzahl COUNT(p.pers_id)
            FROM pers_pruefer p
            WHERE p.pensionierung_datum IS NULL;
      ELSE
         SELECT INTO anzahl COUNT(p.pers_id)
            FROM pers_pruefer p
            WHERE to_char(p.einstellung_datum,''YYYY'') <= to_char($1,''YYYY'') AND
                  (p.pensionierung_datum IS NULL OR
                   to_char(p.pensionierung_datum,''YYYY'') >= to_char($1,''YYYY''));
      END IF;
      RETURN anzahl;
   END;
' LANGUAGE 'plpgsql';


-- ====== Anzahl Prfer/Studenten-Statistik =================================
DROP VIEW stat_anzahl;
CREATE VIEW stat_anzahl AS
   SELECT NOW()                    AS Heute,
          stat_studenten_anzahl(NULL) AS Studenten,
          stat_pruefer_anzahl(NULL)   AS Pruefer;


-- ====== Tabelle mit Jahreszahlen fr Statistik ============================
DROP TABLE Jahre;
CREATE TABLE Jahre (
   Jahr TIMESTAMP
);

BEGIN TRANSACTION;
INSERT INTO Jahre VALUES (to_timestamp('1985','YYYY'));
INSERT INTO Jahre VALUES (to_timestamp('1986','YYYY'));
INSERT INTO Jahre VALUES (to_timestamp('1987','YYYY'));
INSERT INTO Jahre VALUES (to_timestamp('1988','YYYY'));
INSERT INTO Jahre VALUES (to_timestamp('1989','YYYY'));
INSERT INTO Jahre VALUES (to_timestamp('1990','YYYY'));
INSERT INTO Jahre VALUES (to_timestamp('1991','YYYY'));
INSERT INTO Jahre VALUES (to_timestamp('1992','YYYY'));
INSERT INTO Jahre VALUES (to_timestamp('1993','YYYY'));
INSERT INTO Jahre VALUES (to_timestamp('1994','YYYY'));
INSERT INTO Jahre VALUES (to_timestamp('1995','YYYY'));
INSERT INTO Jahre VALUES (to_timestamp('1996','YYYY'));
INSERT INTO Jahre VALUES (to_timestamp('1997','YYYY'));
INSERT INTO Jahre VALUES (to_timestamp('1998','YYYY'));
INSERT INTO Jahre VALUES (to_timestamp('1999','YYYY'));
INSERT INTO Jahre VALUES (to_timestamp('2000','YYYY'));
INSERT INTO Jahre VALUES (to_timestamp('2001','YYYY'));
END TRANSACTION;


-- ====== Anzahl der Vordiplome eines Jahres ================================
DROP FUNCTION stat_vd_anzahl(TIMESTAMP);
CREATE FUNCTION stat_vd_anzahl(TIMESTAMP) RETURNS INTEGER AS '
   DECLARE
      anzahl INTEGER;
   BEGIN
      SELECT INTO anzahl COUNT(q.pers_id)
         FROM uni_qualifikate q
         WHERE q.qal_typ = ''VDIPL'' AND
               to_char(q.datum,''YYYY'') = to_char($1,''YYYY'');
      RETURN anzahl;
   END;
' LANGUAGE 'plpgsql';


-- ====== Anzahl der Diplome eines Jahres ===================================
DROP FUNCTION stat_hd_anzahl(TIMESTAMP);
CREATE FUNCTION stat_hd_anzahl(TIMESTAMP) RETURNS INTEGER AS '
   DECLARE
      anzahl INTEGER;
   BEGIN
      SELECT INTO anzahl COUNT(q.pers_id)
         FROM uni_qualifikate q
         WHERE q.qal_typ = ''DIPLOM'' AND
               to_char(q.datum,''YYYY'') = to_char($1,''YYYY'');
      RETURN anzahl;
   END;
' LANGUAGE 'plpgsql';


-- ====== Leistungsnachweise eines Jahres ===================================
DROP FUNCTION stat_lnw_anzahl(TIMESTAMP);
CREATE FUNCTION stat_lnw_anzahl(TIMESTAMP) RETURNS INTEGER AS '
   DECLARE
      anzahl INTEGER;
   BEGIN
      SELECT INTO anzahl COUNT(q.pers_id)
         FROM uni_qualifikate q
         WHERE q.qal_typ <> ''VDIPL''  AND
               q.qal_typ <> ''DIPLOM'' AND
               q.qal_typ <> ''HZB''    AND
               to_char(q.datum,''YYYY'') = to_char($1,''YYYY'');
      RETURN anzahl;
   END;
' LANGUAGE 'plpgsql';


-- ====== Anzahl immatrikulierter Studenten (weiblich) ======================
DROP FUNCTION stat_studenten_w_anzahl(TIMESTAMP);
CREATE FUNCTION stat_studenten_w_anzahl(TIMESTAMP) RETURNS INTEGER AS '
   DECLARE
      anzahl INTEGER;
   BEGIN
      SELECT INTO anzahl COUNT(s.stud_id)
         FROM pers_studenten s
         WHERE s.m_w = ''w''                                           AND
               to_char(s.immat_datum,''YYYY'') <= to_char($1,''YYYY'') AND
               (s.exmat_datum IS NULL OR
                to_char(s.exmat_datum,''YYYY'') >= to_char($1,''YYYY''));
      RETURN anzahl;
   END;
' LANGUAGE 'plpgsql';


-- ====== Anzahl der nicht-pensionierten Prfer (weiblich) ==================
DROP FUNCTION stat_pruefer_w_anzahl(TIMESTAMP);
CREATE FUNCTION stat_pruefer_w_anzahl(TIMESTAMP) RETURNS INTEGER AS '
   DECLARE
      anzahl INTEGER;
   BEGIN
      SELECT INTO anzahl COUNT(p.pers_id)
         FROM pers_pruefer p
         WHERE p.m_w = ''w''                                                 AND
               to_char(p.einstellung_datum,''YYYY'') <= to_char($1,''YYYY'') AND
               (p.pensionierung_datum IS NULL OR
                to_char(p.pensionierung_datum,''YYYY'') >= to_char($1,''YYYY''));
      RETURN anzahl;
   END;
' LANGUAGE 'plpgsql';


-- ====== Lnge der Studienzeit =============================================
DROP FUNCTION stat_studenten_szeit(TIMESTAMP);
CREATE FUNCTION stat_studenten_szeit(TIMESTAMP) RETURNS FLOAT AS '
   DECLARE
      anzahl FLOAT;
   BEGIN
      SELECT INTO anzahl ROUND(AVG(stat_studenten_semester(s.stud_id)),2)
         FROM pers_studenten s
         WHERE to_char(s.immat_datum,''YYYY'') <= to_char($1,''YYYY'') AND
               s.exmat_datum IS NOT NULL                               AND
               to_char(s.exmat_datum,''YYYY'') >= to_char($1,''YYYY'');
      IF anzahl IS NULL THEN
         SELECT INTO anzahl 0;
      END IF;
      RETURN anzahl;
   END;
' LANGUAGE 'plpgsql';


-- ====== Hilfs-Views =======================================================
DROP VIEW stat_jahre_tmp;
CREATE VIEW stat_jahre_tmp AS
   SELECT Jahr,
          stat_pruefer_anzahl(Jahr)     AS Pruefer,
          stat_pruefer_w_anzahl(Jahr)   AS PrueferW,
          stat_studenten_anzahl(Jahr)   AS Studenten,
          stat_studenten_w_anzahl(Jahr) AS StudentenW,
          stat_vd_anzahl(Jahr)          AS Vordiplome,
          stat_hd_anzahl(Jahr)          AS Diplome,
          stat_lnw_anzahl(Jahr)         AS Leistnw,
          stat_studenten_szeit(Jahr)    AS Studienzeit
   FROM Jahre
   GROUP BY Jahr;


-- ====== Anzahl Prfer/Studenten-Statistik nach Jahreszahl =================
DROP VIEW stat_jahre;
CREATE VIEW stat_jahre AS
   SELECT to_char(Jahr,'YYYY')                           AS Jahr,
          lpad(Pruefer,4,' ')                            AS Pruefer,
          lpad(PrueferW,4,' ')                           AS PrueferW,
          to_char(prozent(PrueferW,Pruefer),'999.9')     AS Pfq,
          lpad(Studenten,4,' ')                          AS Studenten,
          lpad(StudentenW,4,' ')                         AS StudentenW,
          to_char(prozent(StudentenW,Studenten),'999.9') AS Sfq,
          to_char(Studienzeit,'99.9')                    AS Studienzeit,
          lpad(Leistnw,4,' ')                            AS Leistnw,
          lpad(Vordiplome,4,' ')                         AS Vordiplome,
          lpad(Diplome,4,' ')                            AS Diplome
   FROM stat_jahre_tmp;
